/* robotarmGLUT.c
 * Bras robotis en mouvement avec quatre doigts
 * Touches:
 *	ESC - Quitter
 * 	a - Activation/Arrt de l'antialiasing
 *	p - Mode pause
 *
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 19 juillet 2002
 */

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Caractristiques de la fentre et de l'cran */
#define TITLE "RobotArmGLUT (Vouriot Thierry)"	// titre du programme
#define WINDOW_WIDTH 350	// largeur de la fentre
#define WINDOW_HEIGHT 350	// largeur de la fentre
#define	WINDOW_POS_X 100	// position x de la fentre
#define WINDOW_POS_Y 100	// position y de la fentre
#define FULLSCREEN_WIDTH 640	// largeur en mode plein cran
#define FULLSCREEN_HEIGHT 480	// hauteur en mode plein cran
#define FULLSCREEN_DEPTH 16	// bit par pixels en mode plen cran

/* Dfinitions pour Windows */
#ifdef _WIN32
#include <windows.h>
#pragma comment( lib, "opengl32.lib" )			// recherche de OpenGL32.lib pendant l'dition de liens
#pragma comment( lib, "glu32.lib" )			// recherche de Glu32.lib pendant l'dition de liens
#pragma comment( lib, "glut32.lib" )			// recherche de Glut32.lib pendant l'dition de liens
#endif

/* Dfinition des boolens pour C */
#ifndef bool
#define bool unsigned char
#define true 1 == 1
#define false !true
#endif

#define ESC 27			// code ascii de la touche escape

/* Fullscreen */
static bool g_fullscreen = false;

/* Variable contenant le temps au moment de l'affichage de la
 * dernire image, cet variable permet de baser les
 * mouvements sur le temps et non pas sur la vitesse d'affichage
 * de votre ordinateur */
static int g_timeAtLastFrame = 0;

/* Structure permettant le mouvement d'un membre */
typedef struct sAng {
	float a;	// angle de rotation
	int inc;	// incrmentation (vitesse de rotation)
	float maxi;	// angle maxi
	float mini;	// angle mini
	bool s;		// sens de rotation
} Ang;

/* Tableau contenant les structures pour chaque membre */
#define N 10
static Ang angles[N];
/* Indice de chaque membre dans le tableau */
#define SHOULDER 0 		// paule
#define ELBOW 1			// coude
#define FING_RIGHT_1 2		// Doigt droit 1
#define FING_RIGHT_2 3		// Doigt droit 2
#define FING_LEFT_1 4		// Doigt gauche 1
#define FING_LEFT_2 5		// Doigt gauche 2
#define FING_CENTER_1 6		// Doigt centre 1
#define FING_CENTER_2 7		// Doigt centre 2
#define FING_BOTTOM_1 8		// Doigt bas 1
#define FING_BOTTOM_2 9		// Doigt bas 2

/* Mode pause */
static bool pause = false;

/* Prototypes */
void init(void);
void askFullscreen(void);
void glutDisplay(void);
void glutIdle(void);
void glutReshape(int w, int h);
void glutKeyboard(unsigned char key, int x, int y);
void terminateApplication(void);

/* Main */
int main(int argc, char ** argv)
{	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB);
	#ifdef _WIN32
	askFullscreen();
	#endif
	if (!g_fullscreen) {
		glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
		glutInitWindowPosition(WINDOW_POS_X, WINDOW_POS_Y);
		glutCreateWindow(TITLE);
	}	
	init();
	glutDisplayFunc(glutDisplay);
	glutIdleFunc(glutIdle);
	glutReshapeFunc(glutReshape);
	glutKeyboardFunc(glutKeyboard);
	glutMainLoop();
	return 0;
}

/* Permet de passer en mode plein ecran ou en mode fenetre
 * Utilise GLUT_GAME_MODE disponible uniquement sous Windows
 */
#ifdef _WIN32
void askFullscreen(void)
{
	char gameMode[14];
	
	/* Demande  l'utilisateur s'il veut le mode plein cran */
	if (!g_fullscreen)
	g_fullscreen = (MessageBox(NULL, "Click Yes to go to full screen (Recommended)", "Start FullScreen?",
						MB_ICONQUESTION | MB_YESNO) == IDYES);
	
	
	if (g_fullscreen) {	// game mode (fullscreen)
		sprintf(gameMode, "%dx%d:%d", FULLSCREEN_WIDTH, FULLSCREEN_HEIGHT, FULLSCREEN_DEPTH);
		glutGameModeString(gameMode);
		// teste si le game mode est possible
		if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
		{
			glutEnterGameMode();
			//glutSetCursor(GLUT_CURSOR_NONE);
		}
		else {
			g_fullscreen = false;
		}
	} // mode fentr
}
#endif

/* Initialisation */
void init(void)
{
	/* Affiche des informations sur OpenGL */
	char *exten;
	exten = (char *) glGetString(GL_VENDOR);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_RENDERER);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_VERSION);
	printf("%s\n",exten);
	
	printf("\nTouches :\n");
 	printf("\ta - Activation/Arret de l'antialiasing\n");
 	printf("\tp - Mode pause\n");
	printf("\tESC - Quitter\n");
	
	g_timeAtLastFrame = glutGet(GLUT_ELAPSED_TIME);
	
	/* Remplissage des structures pour chaque membre */
	angles[SHOULDER].a = 0.0;
	angles[SHOULDER].inc = 70;
	angles[SHOULDER].maxi = 30.0;
	angles[SHOULDER].mini = 0.0;
	angles[SHOULDER].s = true;
	
	angles[ELBOW].a = 0.0;
	angles[ELBOW].inc = 60;
	angles[ELBOW].maxi = 75.0;
	angles[ELBOW].mini = 0.0;
	angles[ELBOW].s = true;
	
	angles[FING_RIGHT_1].a = 0.0;
	angles[FING_RIGHT_1].inc = 45;
	angles[FING_RIGHT_1].maxi = 55.0;
	angles[FING_RIGHT_1].mini = 0.0;
	angles[FING_RIGHT_1].s = true;
	
	angles[FING_RIGHT_2].a = 0.0;
	angles[FING_RIGHT_2].inc = 30;
	angles[FING_RIGHT_2].maxi = 0.0;
	angles[FING_RIGHT_2].mini = -65.0;
	angles[FING_RIGHT_2].s = true;
	
	angles[FING_LEFT_1].a = 0.0;
	angles[FING_LEFT_1].inc = 30;
	angles[FING_LEFT_1].maxi = 55.0;
	angles[FING_LEFT_1].mini = 0.0;
	angles[FING_LEFT_1].s = true;
	
	angles[FING_LEFT_2].a = 0.0;
	angles[FING_LEFT_2].inc = 30;
	angles[FING_LEFT_2].maxi = 0.0;
	angles[FING_LEFT_2].mini = -65.0;
	angles[FING_LEFT_2].s = true;
	
	angles[FING_CENTER_1].a = 0.0;
	angles[FING_CENTER_1].inc = 25;
	angles[FING_CENTER_1].maxi = 55.0;
	angles[FING_CENTER_1].mini = 0.0;
	angles[FING_CENTER_1].s = true;
	
	angles[FING_CENTER_2].a = 0.0;
	angles[FING_CENTER_2].inc = 35;
	angles[FING_CENTER_2].maxi = 0.0;
	angles[FING_CENTER_2].mini = -65.0;
	angles[FING_CENTER_2].s = true;
	
	angles[FING_BOTTOM_1].a = 0.0;
	angles[FING_BOTTOM_1].inc = 60;
	angles[FING_BOTTOM_1].maxi = 0.0;
	angles[FING_BOTTOM_1].mini = -55.0;
	angles[FING_BOTTOM_1].s = true;
	
	angles[FING_BOTTOM_2].a = 0.0;
	angles[FING_BOTTOM_2].inc = 30;
	angles[FING_BOTTOM_2].maxi = 65.0;
	angles[FING_BOTTOM_2].mini = 0.0;
	angles[FING_BOTTOM_2].s = true;
	
	/* Initialisation de OpenGL */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_FLAT);	
}
		
/* Affichage et animation */
void glutDisplay(void)
{
	glClear(GL_COLOR_BUFFER_BIT);
	glColor3f(1.0, 1.0, 1.0);
	
	glPushMatrix();
		// Epaule
		glTranslatef(-1.0, 0.0, 0.0);
		glRotatef((GLfloat) angles[SHOULDER].a, 0.0, 0.0, 1.0);
		glTranslatef(1.0, 0.0, 0.0);
		
		glPushMatrix();
			glScalef(2.0, 0.5, 0.5);
			glutWireCube(1.0);
		glPopMatrix();
		
		// Coude
		glTranslatef(1.0, 0.25, 0.0);
		glRotatef((GLfloat) angles[ELBOW].a, 0.0, 0.0, 1.0);
		glTranslatef(1.0, -0.25, 0.0);
		
		glPushMatrix();
			glScalef(2.0, 0.5, 0.5);
			glutWireCube(1.0);
		glPopMatrix();
		
		// Doigt centre
		glPushMatrix();
			glTranslatef(1.0, 0.25, 0.0);
			glRotatef((GLfloat) angles[FING_CENTER_1].a, 0.0, 0.0, 1.0);
			glTranslatef(0.2, -0.15/2, 0.0);
		
			glPushMatrix();
				glScalef(0.4, 0.15, 0.15);
				glutWireCube(1.0);
			glPopMatrix();
			
			glTranslatef(0.2, -0.15/2, 0.0);
			glRotatef((GLfloat) angles[FING_CENTER_2].a, 0.0, 0.0, 1.0);
			glTranslatef(0.2, 0.15/2, 0.0);
		
			glPushMatrix();
				glScalef(0.4, 0.15, 0.15);
				glutWireCube(1.0);
			glPopMatrix();
		glPopMatrix();
		
		// Doigt droit
		glPushMatrix();
			glTranslatef(1.0, 0.25, (0.25-0.15/2));
			glRotatef((GLfloat) angles[FING_RIGHT_1].a, 0.0, 0.0, 1.0);
			glTranslatef(0.2, -0.15/2, 0.0);
		
			glPushMatrix();
				glScalef(0.4, 0.15, 0.15);
				glutWireCube(1.0);
			glPopMatrix();
			
			glTranslatef(0.2, -0.15/2, 0.0);
			glRotatef((GLfloat) angles[FING_RIGHT_2].a, 0.0, 0.0, 1.0);
			glTranslatef(0.2, 0.15/2, 0.0);
		
			glPushMatrix();
				glScalef(0.4, 0.15, 0.15);
				glutWireCube(1.0);
			glPopMatrix();
		glPopMatrix();
		
		// Doigt gauche
		glPushMatrix();
			glTranslatef(1.0, 0.25, -(0.25-0.15/2));
			glRotatef((GLfloat) angles[FING_LEFT_1].a, 0.0, 0.0, 1.0);
			glTranslatef(0.2, -0.15/2, 0.0);
		
			glPushMatrix();
				glScalef(0.4, 0.15, 0.15);
				glutWireCube(1.0);
			glPopMatrix();
			
			glTranslatef(0.2, -0.15/2, 0.0);
			glRotatef((GLfloat) angles[FING_LEFT_2].a, 0.0, 0.0, 1.0);
			glTranslatef(0.2, 0.15/2, 0.0);
		
			glPushMatrix();
				glScalef(0.4, 0.15, 0.15);
				glutWireCube(1.0);
			glPopMatrix();
		glPopMatrix();
		
		// Doigt bas
		glPushMatrix();
			glTranslatef(1.0, -0.25, (0.25-0.15/2));
			glRotatef((GLfloat) angles[FING_BOTTOM_1].a, 0.0, 0.0, 1.0);
			glTranslatef(0.2, 0.15/2, 0.0);
		
			glPushMatrix();
				glScalef(0.4, 0.15, 0.15);
				glutWireCube(1.0);
			glPopMatrix();
			
			glTranslatef(0.2, 0.15/2, 0.0);
			glRotatef((GLfloat) angles[FING_BOTTOM_2].a, 0.0, 0.0, 1.0);
			glTranslatef(0.2, -0.15/2, 0.0);
		
			glPushMatrix();
				glScalef(0.4, 0.15, 0.15);
				glutWireCube(1.0);
			glPopMatrix();
		glPopMatrix();
	glPopMatrix();
	
	glFlush();
	glutSwapBuffers();
}

/* Fonction excut lorsque le programme n'a plus rien  faire */
void glutIdle(void)
{
	int i;
	
	int currentTime = glutGet(GLUT_ELAPSED_TIME);
	int frameInterval = currentTime - g_timeAtLastFrame; // Temps coul entre l'affichage de 2 images
	g_timeAtLastFrame = currentTime;

	/* Incrmentation des angles pour chaque structure du tableau */
	if (!pause)
	{
		for (i=0 ; i<N ; i++) {
			if (angles[i].s)
				angles[i].a += (float)frameInterval / angles[i].inc;
			else
				angles[i].a -= (float)frameInterval / angles[i].inc;
			
			// si l'angle maxi ou mini est atteint, on change de sens
			if (angles[i].a >= angles[i].maxi)
				angles[i].s = false;
			if (angles[i].a <= angles[i].mini)
				angles[i].s = true;
		}
		glutPostRedisplay();
	}
}

/* Redimensionnement de la fentre */
void glutReshape(int w, int h)
{
	if (h == 0) h = 1;
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(65.0, (GLfloat)w/(GLfloat)h, 1.0, 20.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	// Placement de l'angle de vue
	glTranslatef(-1.6, -1.7, -5.2);
	glRotatef(-30.0, 0.0, 1.0, 0.0);
}

/* Gestion du clavier */
void glutKeyboard(unsigned char key, int x, int y)
{
	static bool antialias = false;    // antialiasing
	switch(key) {
		case ESC:	// touche escape
			terminateApplication();
			break;
		case 'A':
		case 'a':
			antialias = !antialias;

          		// activation de l'antialiasing
          		if (antialias)
          		{
            			glEnable(GL_LINE_SMOOTH);
            			glEnable(GL_BLEND);
          		}
          		// arrt de l'antialiasing
          		else
          		{
            			glDisable(GL_LINE_SMOOTH);
            			glDisable(GL_BLEND);
          		}
			break;
		case 'P':
		case 'p':	// mode pause
			pause = !pause;
			break;
	}
}

/* Termine l'application */
void terminateApplication(void)
{
	#ifdef _WIN32
	if (g_fullscreen) glutLeaveGameMode();
	#endif
	printf("\nAuteur: Thierry Vouriot\nhttp://thierry.vouriot.free.fr\n");
	exit(1);
}



